<?php

namespace Drupal\Tests\config_overlay\Functional;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\user\RoleInterface;

/**
 * Tests installation of the Minimal profile with Configuration Overlay.
 *
 * @group config_overlay
 */
class ConfigOverlayStandardTest extends ConfigOverlayMinimalTest {

  /**
   * {@inheritdoc}
   */
  protected $profile = 'standard';

  /**
   * {@inheritdoc}
   */
  protected function getOverriddenConfig() {
    $overridden_config = parent::getOverriddenConfig();

    // Add any modules that are installed by Standard, but not by Minimal.
    $overridden_config['core.extension']['module'] += [
      'automated_cron' => 0,
      'big_pipe' => 0,
      'block_content' => 0,
      'breakpoint' => 0,
      'ckeditor' => 0,
      'color' => 0,
      'comment' => 0,
      'config' => 0,
      'contact' => 0,
      'contextual' => 0,
      'datetime' => 0,
      'editor' => 0,
      'field_ui' => 0,
      'file' => 0,
      'help' => 0,
      'history' => 0,
      'image' => 0,
      'link' => 0,
      'menu_ui' => 0,
      'options' => 0,
      'path' => 0,
      'quickedit' => 0,
      'rdf' => 0,
      'search' => 0,
      'shortcut' => 0,
      'taxonomy' => 0,
      'toolbar' => 0,
      'tour' => 0,
      'views_ui' => 0,
      'menu_link_content' => 1,
      'views' => 10,
    ];
    $overridden_config['core.extension']['module'] = module_config_sort($overridden_config['core.extension']['module']);

    $overridden_config['core.extension']['theme'] = [
      'stable' => 0,
      'classy' => 0,
      'bartik' => 0,
      'seven' => 0,
    ];

    // Add text formats. Because the shipped configuration includes a 'roles'
    // property which is not persisted, they will always be considered
    // overridden.
    foreach (['basic_html', 'full_html', 'restricted_html'] as $format_id) {
      $overridden_config["filter.format.$format_id"] = [];
    }

    // Add optional configuration.
    $overridden_config['rdf.mapping.user.user'] = $this->readOptionalConfig('user', 'rdf.mapping.user.user');
    $overridden_config['search.page.node_search'] = $this->readOptionalConfig('node', 'search.page.node_search');
    $overridden_config['search.page.user_search'] = $this->readOptionalConfig('user', 'search.page.user_search');
    $overridden_config['tour.tour.views-ui'] = $this->readOptionalConfig('views_ui', 'tour.tour.views-ui');
    $overridden_config['views.view.archive'] = $this->readOptionalConfig('node', 'views.view.archive');
    $overridden_config['views.view.block_content'] = $this->readOptionalConfig('block_content', 'views.view.block_content');
    $overridden_config['views.view.comment'] = $this->readOptionalConfig('comment', 'views.view.comment');
    $overridden_config['views.view.comments_recent'] = $this->readOptionalConfig('comment', 'views.view.comments_recent');
    $overridden_config['views.view.content'] = $this->readOptionalConfig('node', 'views.view.content');
    $overridden_config['views.view.content_recent'] = $this->readOptionalConfig('node', 'views.view.content_recent');
    $overridden_config['views.view.files'] = $this->readOptionalConfig('file', 'views.view.files');
    $overridden_config['views.view.frontpage'] = $this->readOptionalConfig('node', 'views.view.frontpage');
    $overridden_config['views.view.glossary'] = $this->readOptionalConfig('node', 'views.view.glossary');
    $overridden_config['views.view.taxonomy_term'] = $this->readOptionalConfig('taxonomy', 'views.view.taxonomy_term');
    $overridden_config['views.view.user_admin_people'] = $this->readOptionalConfig('user', 'views.view.user_admin_people');
    $overridden_config['views.view.watchdog'] = $this->readOptionalConfig('dblog', 'views.view.watchdog');
    $overridden_config['views.view.who_s_new'] = $this->readOptionalConfig('user', 'views.view.who_s_new');
    $overridden_config['views.view.who_s_online'] = $this->readOptionalConfig('user', 'views.view.who_s_online');

    /* @see https://www.drupal.org/project/drupal/issues/2989007 */
    $overridden_config['views.view.block_content']['display']['default']['cache_metadata']['max-age'] = CacheBackendInterface::CACHE_PERMANENT;
    $overridden_config['views.view.block_content']['display']['page_1']['cache_metadata']['max-age'] = CacheBackendInterface::CACHE_PERMANENT;
    $overridden_config['views.view.files']['display']['default']['cache_metadata']['max-age'] = CacheBackendInterface::CACHE_PERMANENT;
    $overridden_config['views.view.files']['display']['page_1']['cache_metadata']['max-age'] = CacheBackendInterface::CACHE_PERMANENT;
    $overridden_config['views.view.files']['display']['page_2']['cache_metadata']['max-age'] = CacheBackendInterface::CACHE_PERMANENT;
    $overridden_config['views.view.glossary']['display']['default']['cache_metadata']['max-age'] = CacheBackendInterface::CACHE_PERMANENT;
    $overridden_config['views.view.glossary']['display']['attachment_1']['cache_metadata']['max-age'] = CacheBackendInterface::CACHE_PERMANENT;
    $overridden_config['views.view.glossary']['display']['page_1']['cache_metadata']['max-age'] = CacheBackendInterface::CACHE_PERMANENT;
    $overridden_config['views.view.watchdog']['display']['default']['cache_metadata']['max-age'] = CacheBackendInterface::CACHE_PERMANENT;
    $overridden_config['views.view.watchdog']['display']['page']['cache_metadata']['max-age'] = CacheBackendInterface::CACHE_PERMANENT;

    /* @see user_user_role_insert() */
    $overridden_config['system.action.user_add_role_action.administrator'] = $this->processConfigEntityData('system.action.user_add_role_action.administrator', [
      'langcode' => 'en',
      'status' => TRUE,
      'dependencies' => [
        'config' => ['user.role.administrator'],
        'module' => ['user'],
      ],
      'id' => 'user_add_role_action.administrator',
      'label' => 'Add the Administrator role to the selected user(s)',
      'type' => 'user',
      'plugin' => 'user_add_role_action',
      'configuration' => [
        'rid' => 'administrator',
      ],
    ]);
    $overridden_config['system.action.user_remove_role_action.administrator'] = $this->processConfigEntityData('system.action.user_remove_role_action.administrator', [
      'langcode' => 'en',
      'status' => TRUE,
      'dependencies' => [
        'config' => ['user.role.administrator'],
        'module' => ['user'],
      ],
      'id' => 'user_remove_role_action.administrator',
      'label' => 'Remove the Administrator role from the selected user(s)',
      'type' => 'user',
      'plugin' => 'user_remove_role_action',
      'configuration' => [
        'rid' => 'administrator',
      ],
    ]);

    /* @see shortcut_themes_installed() */
    $overridden_config['seven.settings'] = [
      'third_party_settings' => [
        'shortcut' => ['module_link' => TRUE],
      ],
    ];

    /* @see standard_install() */
    $overridden_config['system.site']['page']['front'] = '/node';
    $overridden_config['user.settings'] = ['register' => USER_REGISTER_VISITORS_ADMINISTRATIVE_APPROVAL];
    // Add menu link overrides.
    $overridden_config['core.menu.static_menu_link_overrides'] = [
      'definitions' => [
        'contact__site_page' => [
          'enabled' => TRUE,
          'menu_name' => 'footer',
          'parent' => '',
          'weight' => 0,
          'expanded' => FALSE,
        ],
      ],
    ];
    // Add permissions common to both anonymous and authenticated users.
    $role_ids = [RoleInterface::ANONYMOUS_ID, RoleInterface::AUTHENTICATED_ID];
    foreach ($role_ids as $role_id) {
      $overridden_config["user.role.$role_id"]['permissions'] = array_merge(
        $overridden_config["user.role.$role_id"]['permissions'],
        ['access comments', 'access site-wide contact form', 'search content']
      );
    }
    // Add role-specific permissions.
    $overridden_config['user.role.' . RoleInterface::ANONYMOUS_ID]['permissions'] = array_merge(
      $overridden_config['user.role.' . RoleInterface::ANONYMOUS_ID]['permissions'],
      ['use text format restricted_html']
    );
    $overridden_config['user.role.' . RoleInterface::AUTHENTICATED_ID]['permissions'] = array_merge(
      $overridden_config['user.role.' . RoleInterface::AUTHENTICATED_ID]['permissions'],
      ['access shortcuts', 'post comments', 'skip comment approval', 'use text format basic_html']
    );
    sort($overridden_config['user.role.' . RoleInterface::ANONYMOUS_ID]['permissions']);
    sort($overridden_config['user.role.' . RoleInterface::AUTHENTICATED_ID]['permissions']);

    $overridden_config['node.settings'] = ['use_admin_theme' => TRUE];

    /* @see standard_form_install_configure_submit() */
    $overridden_config['contact.form.feedback'] = [
      'recipients' => ['simpletest@example.com'],
    ];

    return $overridden_config;
  }

  /**
   * {@inheritdoc}
   */
  protected function getExpectedConfig() {
    $expected_config = parent::getExpectedConfig();

    foreach (['basic_html', 'full_html', 'restricted_html'] as $format_id) {
      unset($expected_config["filter.format.$format_id"]['roles']);
    }

    return $expected_config;
  }

}
