<?php

namespace Drupal\Tests\config_overlay\Functional;

use Drupal\language\Entity\ConfigurableLanguage;

/**
 * Tests installation of the Testing profile with Configuration Overlay.
 *
 * @group config_overlay
 */
class ConfigOverlayTestingLanguageTest extends ConfigOverlayTestingTest {

  /**
   * The language to install the site in.
   *
   * @var string
   */
  protected $langcode = 'af';

  /**
   * {@inheritdoc}
   */
  protected function installParameters() {
    $parameters = parent::installParameters();
    $parameters['parameters']['langcode'] = $this->langcode;
    return $parameters;
  }

  /**
   * {@inheritdoc}
   */
  protected function getOverriddenConfig() {
    $overridden_config = parent::getOverriddenConfig();

    // Installing in a language other than English enables the Interface
    // Translation mdoule (including its dependencies).
    $overridden_config['core.extension']['module'] += [
      'field' => 0,
      'file' => 0,
      'language' => 0,
      'locale' => 0,
    ];
    $overridden_config['core.extension']['module'] = module_config_sort($overridden_config['core.extension']['module']);

    /* @see drupal_install_system() */
    $overridden_config['system.site']['default_langcode'] = $this->langcode;

    /* @see install_download_additional_translations_operations() */
    /* @see @see \Drupal\Core\Config\Entity\ConfigEntityType::getPropertiesToExport() */
    $language = ConfigurableLanguage::createFromLangcode($this->langcode);
    $config_name = 'language.entity.' . $this->langcode;
    $overridden_config[$config_name] = [
      'uuid' => $this->configStorage->read($config_name)['uuid'],
      'langcode' => $this->langcode,
      'status' => TRUE,
      'dependencies' => [],
      'id' => $this->langcode,
      'label' => $language->label(),
      'direction' => $language->getDirection(),
      'weight' => 0,
      'locked' => FALSE,
    ];
    /* @see \Drupal\language\Entity\ConfigurableLanguage::postSave() */
    $overridden_config['language.negotiation'] = [
      'url' => [
        'prefixes' => [$this->langcode => ''],
        'domains' => [$this->langcode => ''],
      ],
    ];

    // Add optional configuration.
    $overridden_config['locale.settings'] = $this->readOptionalConfig($this->profile, 'locale.settings');
    /* @see locale_install() */
    $overridden_config['locale.settings']['translation']['path'] = $this->siteDirectory . '/files/translations';

    // Add translations of shipped configuration that will be imported upon
    // installation.
    $overridden_config['system.menu.admin'] = [
      'label' => 'Administrasie',
    ];

    // All configuration will specify the site default language as their its
    // language while the shipped configuration specifies English.
    foreach ($this->configStorage->listAll() as $config_name) {
      $overridden_config[$config_name]['langcode'] = $this->langcode;
    }

    return $overridden_config;
  }

  /**
   * {@inheritdoc}
   */
  protected function getExpectedConfig() {
    $expected_config = parent::getExpectedConfig();

    unset(
      $expected_config['language.negotiation']['url']['prefixes']['en'],
      $expected_config['language.negotiation']['url']['domains']['en']
    );

    /* @see https://www.drupal.org/project/drupal/issues/2988960 */
    unset($expected_config['locale.settings']['translate_english']);

    return $expected_config;
  }

}
